@@ -137,6 +137,56 @@ _gda_sqlite_provider_meta_init (GdaServerProvider *provider)
 	g_static_mutex_unlock (&init_mutex);
 }
 
+static GdaStatement *
+get_statement (InternalStatementItem type, const gchar *schema_name, const gchar *obj_name, GError **error)
+{
+	GdaStatement *stmt;
+	if (strcmp (schema_name, "main")) {
+		gchar *str;
+		
+		switch (type) {
+		case I_PRAGMA_TABLE_INFO:
+			str = g_strdup_printf ("PRAGMA %s.table_info (%s)", schema_name, obj_name);
+			break;
+		case I_PRAGMA_INDEX_LIST:
+			str = g_strdup_printf ("PRAGMA %s.index_list (%s)", schema_name, obj_name);
+			break;
+		case I_PRAGMA_INDEX_INFO:
+			str = g_strdup_printf ("PRAGMA %s.index_info (%s)", schema_name, obj_name);
+			break;
+		case I_PRAGMA_FK_LIST:
+			str = g_strdup_printf ("PRAGMA %s.foreign_key_list (%s)", schema_name, obj_name);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+		
+		stmt = gda_sql_parser_parse_string (internal_parser, str, NULL, NULL);
+		g_free (str);
+		g_assert (stmt);
+	}
+	else {
+		switch (type) {
+		case I_PRAGMA_TABLE_INFO:
+		case I_PRAGMA_INDEX_LIST:
+		case I_PRAGMA_FK_LIST:
+			if (! gda_set_set_holder_value (pragma_set, error, "tblname", obj_name))
+				return NULL;
+			break;
+		case I_PRAGMA_INDEX_INFO:
+			if (! gda_set_set_holder_value (pragma_set, error, "idxname", obj_name))
+				return NULL;
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+
+		stmt = g_object_ref (internal_stmt [type]);
+	}
+
+	return stmt;
+}
+
 gboolean
 _gda_sqlite_meta__info (GdaServerProvider *prov, GdaConnection *cnc, 
 			GdaMetaStore *store, GdaMetaContext *context, GError **error)
@@ -172,7 +222,7 @@ _gda_sqlite_meta__btypes (GdaServerProvider *prov, GdaConnection *cnc,
 		{"integer", "gint", "Signed integer, stored in 1, 2, 3, 4, 6, or 8 bytes depending on the magnitude of the value", "int"},
 		{"real", "gdouble",  "Floating point value, stored as an 8-byte IEEE floating point number", NULL},
 		{"text", "string", "Text string, stored using the database encoding", "string"},
-		{"blob", "GdaBinary", "Blob of data, stored exactly as it was input", NULL},
+		{"blob", "GdaBlob", "Blob of data, stored exactly as it was input", NULL},
 		{"timestamp", "GdaTimestamp", "Time stamp, stored as 'YYYY-MM-DD HH:MM:SS.SSS'", NULL},
 		{"time", "GdaTime", "Time, stored as 'HH:MM:SS.SSS'", NULL},
 		{"date", "GDate", "Date, stored as 'YYYY-MM-DD'", NULL},
@@ -278,7 +328,7 @@ fill_udt_model (SqliteConnectionData *cdata, GHashTable *added_hash,
 		gint fields_status;
 
 		if (strcmp (cstr, "main")) 
-			sql = g_strdup_printf ("PRAGMA table_info('%s.%s');", cstr, sqlite3_column_text (tables_stmt, 0));
+			sql = g_strdup_printf ("PRAGMA %s.table_info(%s);", cstr, sqlite3_column_text (tables_stmt, 0));
 		else
 			sql = g_strdup_printf ("PRAGMA table_info('%s');", sqlite3_column_text (tables_stmt, 0));
 		fields_status = sqlite3_prepare_v2 (cdata->connection, sql, -1, &fields_stmt, NULL);
@@ -738,7 +788,7 @@ _gda_sqlite_meta__tables_views (GdaServerProvider *prov, GdaConnection *cnc,
 			break;
 		}
 		if (!strcmp (g_value_get_string (cvalue), TMP_DATABASE_NAME))
-			 continue; /* nothing to do */
+			continue; /* nothing to do */
 
 		if (! fill_tables_views_model (cnc, tables_model, views_model, 
 					       cvalue, NULL, error)) {
@@ -808,25 +858,14 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 	gint i;
 	GType col_types[] = {G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, 
 			     G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_NONE};
+	GdaStatement *stmt;
 	
 	schema_name = g_value_get_string (p_table_schema);
-	if (strcmp (schema_name, "main")) {
-		gchar *str;
-		str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (p_table_name));
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", str)) {
-			g_free (str);
-			return FALSE;
-		}
-		g_free (str);
-	}
-	else {
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", g_value_get_string (p_table_name)))
-			return FALSE;
-	}
-
-	tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_TABLE_INFO], pragma_set, 
+	stmt = get_statement (I_PRAGMA_TABLE_INFO, schema_name, g_value_get_string (p_table_name), error);
+	tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 								 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 								 col_types, error);
+	g_object_unref (stmt);
 	if (!tmpmodel)
 		return FALSE;
 		
@@ -979,7 +1018,7 @@ _gda_sqlite_meta__columns (GdaServerProvider *prov, GdaConnection *cnc,
 		}
 		schema_name = g_value_get_string (cvalue); 
 		if (!strcmp (schema_name, TMP_DATABASE_NAME))
-			 continue; /* nothing to do */
+			continue; /* nothing to do */
 		
 		gchar *str;
 		GType col_types[] = {G_TYPE_STRING, G_TYPE_STRING, G_TYPE_NONE};
@@ -1078,24 +1117,9 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	gint nrows;
 	const gchar *schema_name;
 	gint i;
+	GdaStatement *stmt;
 
-	/*
-	 * Setup pragma_set
-	 */
 	schema_name = g_value_get_string (p_table_schema);
-	if (strcmp (schema_name, "main")) {
-		gchar *str;
-		str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (p_table_name));
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", str)) {
-			g_free (str);
-			return FALSE;
-		}
-		g_free (str);
-	}
-	else {
-		if (!gda_set_set_holder_value (pragma_set, error, "tblname", g_value_get_string (p_table_name)))
-			return FALSE;
-	}	
 
 	/* 
 	 * PRIMARY KEY
@@ -1106,9 +1130,11 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 				G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_NONE};
 	gboolean has_pk = FALSE;
 
-	tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_TABLE_INFO], pragma_set, 
+	stmt = get_statement (I_PRAGMA_TABLE_INFO, schema_name, g_value_get_string (p_table_name), error);
+	tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 								 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 								 pk_col_types, error);
+	g_object_unref (stmt);
 	if (!tmpmodel)
 		return FALSE;
 		
@@ -1183,9 +1209,11 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	 */
 	GType unique_col_types[] = {G_TYPE_INT, G_TYPE_STRING, G_TYPE_INT, G_TYPE_NONE};
 	
-	tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_INDEX_LIST], pragma_set, 
+	stmt = get_statement (I_PRAGMA_INDEX_LIST, schema_name, g_value_get_string (p_table_name), error);
+	tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 								 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 								 unique_col_types, error);
+	g_object_unref (stmt);
 	if (!tmpmodel)
 		return FALSE;
 		
@@ -1233,10 +1261,12 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	 * FOREIGN KEYS
 	 */
 	GType fk_col_types[] = {G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_NONE};
-	gchar *ref_table = NULL;
-	tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_FK_LIST], pragma_set, 
+	gint fkid = -1;
+	stmt = get_statement (I_PRAGMA_FK_LIST, schema_name, g_value_get_string (p_table_name), error);
+	tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 								 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 								 fk_col_types, error);
+	g_object_unref (stmt);
 	if (!tmpmodel)
 		return FALSE;
 		
@@ -1244,18 +1274,23 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	for (i = 0; i < nrows; i++) {
 		const GValue *cvalue;
 
-		cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+		cvalue = gda_data_model_get_value_at (tmpmodel, 0, i, error);
 		if (!cvalue) {
 			retval = FALSE;
 			break;
 		}
-		if (! ref_table || strcmp (ref_table, g_value_get_string (cvalue))) {
+		if ((fkid == -1) || (fkid != g_value_get_int (cvalue))) {
 			gchar *constname;
 			GValue *v1, *v2;
 
-			g_free (ref_table);
-			ref_table = g_strdup (g_value_get_string (cvalue));
-			constname = g_strdup_printf ("fk_%s", ref_table);
+			fkid = g_value_get_int (cvalue);
+
+			cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+			if (!cvalue) {
+				retval = FALSE;
+				break;
+			}
+			constname = g_strdup_printf ("fk%d_%s", fkid, g_value_get_string (cvalue));
 			if (constraint_name_n && strcmp (g_value_get_string (constraint_name_n), constname)) {
 				g_free (constname);
 				continue;
@@ -1278,7 +1313,6 @@ fill_constraints_tab_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 				retval = FALSE;
 		}
 	}
-	g_free (ref_table);
 	g_object_unref (tmpmodel);
 
 	/*
@@ -1403,30 +1437,20 @@ fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	gint nrows;
 	const gchar *schema_name;
 	gint i;
+	GdaStatement *stmt;
 
 	/*
-	 * Setup pragma_set
+	 * Setup stmt to execute
 	 */
 	schema_name = g_value_get_string (p_table_schema);
-	if (strcmp (schema_name, "main")) {
-		gchar *str;
-		str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (p_table_name));
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", str)) {
-			g_free (str);
-			return FALSE;
-		}
-		g_free (str);
-	}
-	else {
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", g_value_get_string (p_table_name)))
-			return FALSE;
-	}
 
 	GType fk_col_types[] = {G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_NONE};
-	gchar *ref_table = NULL;
-	tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_FK_LIST], pragma_set, 
+	gint fkid = -1;
+	stmt = get_statement (I_PRAGMA_FK_LIST, schema_name, g_value_get_string (p_table_name), error);
+	tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 								 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 								 fk_col_types, error);
+	g_object_unref (stmt);
 	if (!tmpmodel)
 		return FALSE;
 		
@@ -1434,25 +1458,31 @@ fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 	for (i = 0; i < nrows; i++) {
 		const GValue *cvalue;
 
-		cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+		cvalue = gda_data_model_get_value_at (tmpmodel, 0, i, error);
 		if (!cvalue) {
 			retval = FALSE;
 			break;
 		}
-		if (! ref_table || strcmp (ref_table, g_value_get_string (cvalue))) {
+		if ((fkid  == -1) || (fkid != g_value_get_int (cvalue))) {
 			gchar *constname;
 			GValue *v2, *v3, *v4, *v5;
 
-			g_free (ref_table);
-			ref_table = g_strdup (g_value_get_string (cvalue));
-			constname = g_strdup_printf ("fk_%s", ref_table);
+			fkid = g_value_get_int (cvalue);
+
+			cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+			if (!cvalue) {
+				retval = FALSE;
+				break;
+			}
+
+			constname = g_strdup_printf ("fk%d_%s", fkid, g_value_get_string (cvalue));
 			if (constraint_name_n && strcmp (g_value_get_string (constraint_name_n), constname)) {
 				g_free (constname);
 				continue;
 			}
 		
 			g_value_set_string ((v2 = gda_value_new (G_TYPE_STRING)), "FOREIGN KEY");
-			g_value_set_string ((v3 = gda_value_new (G_TYPE_STRING)), ref_table);
+			g_value_set_string ((v3 = gda_value_new (G_TYPE_STRING)), g_value_get_string (cvalue));
 			g_value_set_string ((v4 = gda_value_new (G_TYPE_STRING)), "primary_key");
 			if (!constraint_name_n)
 				g_value_take_string ((v5 = gda_value_new (G_TYPE_STRING)), constname);
@@ -1475,7 +1505,6 @@ fill_constraints_ref_model (GdaConnection *cnc, SqliteConnectionData *cdata, Gda
 				g_free (constname);
 		}
 	}
-	g_free (ref_table);
 	g_object_unref (tmpmodel);
 
 	return retval;
@@ -1595,25 +1624,9 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 	gint nrows;
 	const gchar *schema_name, *const_name;
 	gint i;
+	GdaStatement *stmt;
 
-	/*
-	 * Setup pragma_set
-	 */
 	schema_name = g_value_get_string (p_table_schema);
-	if (strcmp (schema_name, "main")) {
-		gchar *str;
-		str = g_strdup_printf ("%s.%s", schema_name, g_value_get_string (p_table_name));
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", str)) {
-			g_free (str);
-			return FALSE;
-		}
-		g_free (str);
-	}
-	else {
-		if (! gda_set_set_holder_value (pragma_set, error, "tblname", g_value_get_string (p_table_name)))
-			return FALSE;
-	}
-
 	const_name = g_value_get_string (constraint_name);
 	if (!strcmp (const_name, "primary_key")) {
 		/* 
@@ -1622,9 +1635,11 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 		GType pk_col_types[] = {G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, 
 					G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_NONE};
 		gint ord_pos = 1;
-		tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_TABLE_INFO], pragma_set, 
+		stmt = get_statement (I_PRAGMA_TABLE_INFO, schema_name, g_value_get_string (p_table_name), error);
+		tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 									 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 									 pk_col_types, error);
+		g_object_unref (stmt);
 		if (!tmpmodel)
 			return FALSE;
 		
@@ -1684,16 +1699,18 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 		}
 		g_object_unref (tmpmodel);
 	}
-	else if ((*const_name == 'f')  && (const_name[1] == 'k') && (const_name[2] == '_')) {
+	else if ((*const_name == 'f')  && (const_name[1] == 'k')) {
 		/*
 		 * FOREIGN key columns
 		 */
 		GType fk_col_types[] = {G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_NONE};
-		gchar *ref_table = NULL;
+		gint fkid = -1;
 		gint ord_pos;
-		tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_FK_LIST], pragma_set, 
+		stmt = get_statement (I_PRAGMA_FK_LIST, schema_name, g_value_get_string (p_table_name), error);
+		tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 									 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 									 fk_col_types, error);
+		g_object_unref (stmt);
 		if (!tmpmodel)
 			return FALSE;
 		
@@ -1702,21 +1719,25 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 			const GValue *cvalue;
 			GValue *v1;
 			
-			cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+			cvalue = gda_data_model_get_value_at (tmpmodel, 0, i, error);
 			if (!cvalue) {
 				retval = FALSE;
 				break;
 			}
-			if (! ref_table || strcmp (ref_table, g_value_get_string (cvalue))) {
+			if ((fkid == -1) || (fkid != g_value_get_int (cvalue))) {
 				gchar *constname;
 				
-				g_free (ref_table);
-				ref_table = g_strdup (g_value_get_string (cvalue));
-				constname = g_strdup_printf ("fk_%s", ref_table);
-				if (strcmp (g_value_get_string (constraint_name), constname)) {
+				fkid = g_value_get_int (cvalue);
+
+				cvalue = gda_data_model_get_value_at (tmpmodel, 2, i, error);
+				if (!cvalue) {
+					retval = FALSE;
+					break;
+				}
+				constname = g_strdup_printf ("fk%d_%s", fkid, g_value_get_string (cvalue));
+				if (strcmp (const_name, constname)) {
+					fkid = -1;
 					g_free (constname);
-					g_free (ref_table);
-					ref_table = NULL;
 					continue;
 				}
 				ord_pos = 1;
@@ -1737,8 +1758,6 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 					    TRUE, v1 /* ordinal_position */))
 				retval = FALSE;
 		}
-
-		g_free (ref_table);
 		g_object_unref (tmpmodel);
 	}
 	else {
@@ -1747,11 +1766,12 @@ fill_key_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata, GdaData
 		 */
 		GType unique_col_types[] = {G_TYPE_INT, G_TYPE_INT, G_TYPE_STRING, G_TYPE_NONE};
 		
-		if (! gda_set_set_holder_value (pragma_set, error, "idxname", g_value_get_string (constraint_name)))
-			return FALSE;
-		tmpmodel = gda_connection_statement_execute_select_full (cnc, internal_stmt[I_PRAGMA_INDEX_INFO], pragma_set, 
+		stmt = get_statement (I_PRAGMA_INDEX_INFO, schema_name, g_value_get_string (constraint_name),
+				      error);
+		tmpmodel = gda_connection_statement_execute_select_full (cnc, stmt, pragma_set, 
 									 GDA_STATEMENT_MODEL_RANDOM_ACCESS, 
 									 unique_col_types, error);
+		g_object_unref (stmt);
 		if (!tmpmodel)
 			return FALSE;
 		
@@ -1921,7 +1941,7 @@ _gda_sqlite_meta_key_columns (GdaServerProvider *prov, GdaConnection *cnc,
 	g_assert (mod_model);
 
 	retval = fill_key_columns_model (cnc, cdata, mod_model, table_schema, table_name, constraint_name, error);
-	if (retval)
+	if (retval) 
 		retval = gda_meta_store_modify_with_context (store, context, mod_model, error);
 	g_object_unref (mod_model);
 
